/*
    CustRep.CC
    Author...: Ken Mayer
    Date.....: 08/20/1998

    Similar to CustomFormControls.CC in the Vesper system, 
    this contains default custom controls used by the
    author for his reports. The advantage, as with all custom
    controls, is that you can make a single change to all 
    your reports in one place ...

    Controls included at this time:

    kmTitleText1 -- A larger font text object used for a main title
                    on a report
    kmTitleText2 -- A slightly smaller font text object used for
                    a secondary title on a report.
    kmDateText   -- prints "Date:" followed by the current date
    kmFieldHeadingText -- Bold text used to display a field
                        heading
    kmFieldText  -- Used to display actual field, need to set
                    the codeblock appropriately ...
    kmNumFieldText 
                 -- Subclassed from kmFieldText, this
                    displays negative numbers as red ...
    kmRectangleShape 
                 -- unlike the Rectangle object, this one
                    does not shift the text or itself when printed.
    kmLine       -- basic line object, width is set to 2 so it
                    copies better (width of 1 sometimes comes out
                    too thin ...)

    **************************************************************
    Note I have no containers. From Charles Overbeck (on the
    development team for VdBASE 7.x):

    "The container cannot have a band as its parent.  The problem 
    is that the container is currently a windowed control, and 
    we don't allow windowed controls in bands (because the band 
    is potentially rendered multiple times, but there is only 
    one window--so how do you display one window several times?)."
    **************************************************************

*/

/*
    These definitions are used for the text controls below. 
    If you want to change a fontName, fontSize or a colorNormal
    property, do it here:
*/
#define STANDARDFONT     "Arial"
#define STANDARDSIZE     10
#define STANDARDCOLOR    "BLACK"
#define STANDARDNEGCOLOR "RED"
#define CHECKBOXFONT     "Wingdings"
#define CHECKBOXSIZE     12
#define TITLE1FONT       "Arial"
#define TITLE1SIZE       18
#define TITLE1COLOR      "BLACK"
#define TITLE2FONT       "Arial"
#define TITLE2SIZE       14
#define TITLE2COLOR      "BLACK"


/*
   Standard text placed on the pageTemplate, appears as a
   header at the top of each page. 
*/
class kmTitleText1(parentObj) of TEXT(parentObj) custom
   with (this)
      height          := 0.3229
      left            := 1.4792
      top             := 0.0104
      width           := 3.5313
      metric          := 3	// Inches
      alignVertical   := 1	// Middle
      alignHorizontal := 1	// Center
      fontSize        := TITLE1SIZE
      fontName        := TITLE1FONT
      colorNormal     := TITLE1COLOR
      fontBold        := false
      fontItalic      := false
      fontUnderline   := false
      text            := "This is a Main Report Title"
   endwith

endclass

/*
   Note that this is subclassed from above, meant
   to be a smaller item under the first ...
*/
class kmTitleText2(parentObj) of kmTitleText1(parentObj) custom
   with (this)
      fontSize        := TITLE2SIZE
      fontName        := TITLE2FONT
      colorNormal     := TITLE2COLOR
      fontBold        := false
      fontItalic      := false
      fontUnderline   := false
      text            := "This is a secondary title"
   endwith

endclass

/*
   Basic text and date combination ... nice thing about
   VdBASE 7 is the auto casting of types, so that
   we can display the date without having to add the DTOC()
   function ... of course, if you want more detail, like
   character month, and such, you would need to code for
   it ...
*/
class kmDateText(parentObj) of TEXT(parentObj) custom
   with (this)
      height          := 0.2083
      left            := 2.8125
      top             := 0.3229
      width           := 1.1667
      metric          := 3	// Inches
      alignVertical   := 1	// Middle
      text            := {||"Date: "+date()}
      fontSize        := STANDARDSIZE
      fontName        := STANDARDFONT
      colorNormal     := STANDARDCOLOR
      fontBold        := false
      fontItalic      := false
      fontUnderline   := false
   endwith

endclass

/* 
   want the text on this one to be bold and line up on the
   right side ...
*/
class kmFieldHeadingText(parentObj) of TEXT(parentObj) custom
   with (this)
      height          := 0.2083
      left            := 0
      top             := 0.0417
      width           := 0.75
      metric          := 3	// Inches
      alignHorizontal := 2 // Right
      text            := "FieldHeading: "
      fontSize        := STANDARDSIZE
      fontName        := STANDARDFONT
      colorNormal     := STANDARDCOLOR
      fontBold        := true
      fontItalic      := false
      fontUnderline   := false
   endwith

endclass

/*
    FieldText: Suggestion -- place this on the report,
    and then bring up the text property in the codeblock
    editor. Standard text controls are "datalinked" by
    the codeblock:

      form.streamSource1.rowset.fields["fieldName"].value

    So, replace "field codeblock" with the above,
    and of course insert the proper fieldName, and
    you should be fine.

    In addition, you may want to pass the field through
    the string object, particularly if you have
    the potential of nulls, so your codeblock *might* look
    like:

     new String( form.streamSource1.rowset.fields["fieldName"].value )

*/
class kmFieldText(parentObj) of TEXT(parentObj) custom
   with (this)
      height          := 0.1708
      left            := 0.8889
      top             := 0.0424
      width           := 1.2188
      metric          := 3	// Inches
      alignHorizontal := 0 // Left
      variableHeight  := true
      // This needs to be modified for each object ... obviously
      text            := {||"field codeblock"}
      fontSize        := STANDARDSIZE
      fontName        := STANDARDFONT
      colorNormal     := STANDARDCOLOR
      fontBold        := false
      fontItalic      := false
      fontUnderline   := false
   endwith

endclass

/*
     This one is able to change it's color based
     on the value of a numeric field -- if it's negative, 
     we want to show the value in red, otherwise in black ...

     // See commented code below for a bit about the onOpen
     // event, details explained more fully in the kmCheckBoxText
     // class ...
*/
class kmNumFieldText( parentObj ) of kmFieldText( parentObj ) custom
   with (this )
      picture         := "99,999.99" // set appropriate picture
   endwith

   // you may want to set your own onOpen event, that 
   // stores the codeblock you're using for this field,
   // such as (do this using the inspector, go to the
   // events page ...):
   /*
      function kmNumFieldText1_onOpen
         this.text = {|| form.streamSource1.rowset.fields["fieldName"].value }
   */

   function canRender
      // Save codeblock, so we can evaluate it each time:
      if type( "this.text" ) == "CB"
         this.textCB = this.text
      endif

      /* 
         Need to evaluate the codeblock, and if it's numeric
         we need to check for a negative value ...
      */
      if type( "this.textCB()" ) $ "N,F" // numeric or float
         this.colorNormal = iif( this.textCB() < 0, STANDARDNEGCOLOR,;
                                                    STANDARDCOLOR )
      endif

   return true
endclass


/*
   Being able to display a checkbox takes a bit of work
   for those logical fields, but it can be done.
   This text control is subclassed from the standard
   one, rather than the kmFieldText object above,
   because I'll be using a different font, and such.

   Note that the text codeblock needs to evaluate to
   a logical value. If you are using a logical field, store

      form.streamSource1.rowset.fields["fieldName"].value

   in the codeblock editor, replacing "fieldName" with 
   your field name ...

   // *******************************************************
   // Problem: In the designer, sometimes this gets evaluated
   // and the value gets stored that way, and your codeblock
   // gets over-written. You may need to do something in 
   // the onOpen event of the control to store the 
   // codeblock there ... see below the with/endwith block
   // in the code below:
   // ******************************************************
*/
class kmCheckBoxText(parentObj) of Text(parentObj) custom
   with (this)
      height          := 0.1708
      left            := 3.2396
      top             := 0.0528
      width           := 0.1563
      metric          := 3	// Inches
      variableHeight  := false
      fontName        := CHECKBOXFONT
      fontSize        := CHECKBOXSIZE
      colorNormal     := STANDARDCOLOR
      fontBold        := false
      fontItalic      := false
      fontUnderline   := false
      text            := {||"field codeblock" }
   endwith

   // you may want to set your own onOpen event, that 
   // stores the codeblock you're using for this field,
   // such as (do this using the inspector, go to the
   // events page ...):
   /*
      function kmCheckBoxText1_onOpen
         this.text = {|| form.streamSource1.rowset.fields["fieldName"].value }
   */

   function canRender
      // Save codeblock, so we can evaluate it each time:
      if type( "this.text" ) == "CB"
         this.textCB = this.text
      endif

      /* 
         if we get a 'true' out of it, return a checkbox with
         a check in it, otherwise, return an empty checkbox.
         Note that this assumes Wingdings for the font. If you
         want to use a different font, you will need to change
         the characters used ... use the CHARMAP utility
         that ships with some versions of Windows, or other
         Utilities you may find out there ...
      */
      if type( "this.textCB()" ) == "L"
         this.text = iif( this.textCB(), chr(254), chr(168) )
      else
         this.text = "NOT LOGICAL"
      endif

   return true

endclass

/*
   The standard Rectangle doesn't function properly -- this is a
   bug in the software -- if you render the report on screen,
   a rectangle works like you might expect. However, if you
   print it (at least to the several HP Laserjets I've tried
   it with), the rectangle will print offset (up/left) from
   where it should. The shape object, however, doesn't do this --
   so you can generate a rectangle using the shape and have it
   work properly.
*/
class kmRectangleShape(parentObj) of SHAPE(parentObj) custom
   with (this)
      left        := 0.0208
      top         := 0.0208
      width       := 2.9479
      height      := 0.4063
      metric      := 3	// Inches
      colorNormal := "black/white" // border/background
                                   // for a grey rectangle,
                                   // try "silver/silver"
      shapeStyle  := 1	// Rectangle
      penWidth    := 2
   endwith

endclass

/*
    Basic line, but I set the width to 2, because
    when photocopying, I've found that a width
    of 1 often gets lost ...
*/
class kmLine(parentObj) of LINE(parentObj) custom
   with (this)
      left            := 0
      right           := 4.4722
      top             := 0
      bottom          := 0
      width           := 2
      metric          := 3	// Inches
      colorNormal     := STANDARDCOLOR
   endwith

endclass

/*
    ------------------------------------
    End of file: CustomReportControls.CC
    ------------------------------------
*/
